@@ -3,6 +3,8 @@ import os
 from textwrap import dedent
 import sys
 
+from packaging.version import Version
+
 import pytest
 
 import doctest
@@ -12,6 +14,9 @@ from pytest_doctestplus.output_checker import OutputChecker, FLOAT_CMP
 pytest_plugins = ['pytester']
 
 
+PYTEST_LT_6 = Version(pytest.__version__) < Version('6.0.0')
+
+
 def test_ignored_whitespace(testdir):
     testdir.makeini(
         """
@@ -593,6 +598,32 @@ def test_doctest_glob(testdir):
     ).assertoutcome(passed=1)
 
 
+@pytest.mark.xfail(reason='known issue, fenced code blocks require an extra trailing newline')
+def test_markdown_fenced_code(testdir):
+    testdir.makefile('.md', foo="""\
+```
+>>> 1 + 1
+2
+```
+""")
+    testdir.inline_run(
+        '--doctest-plus', '--doctest-glob', '*.md'
+    ).assertoutcome(passed=1)
+
+
+def test_markdown_fenced_code_with_extra_newline(testdir):
+    testdir.makefile('.md', foo="""\
+```
+>>> 1 + 1
+2
+
+```
+""")
+    testdir.inline_run(
+        '--doctest-plus', '--doctest-glob', '*.md'
+    ).assertoutcome(passed=1)
+
+
 def test_text_file_comments(testdir):
     testdir.makefile(
         '.md',
@@ -769,20 +800,38 @@ def test_doctest_float_replacement(tmp_path):
     )
 
 
-def test_doctest_subpackage_requires(testdir, caplog):
-
-    # Note that each entry below has different whitespace around the = to
-    # make sure that all cases work properly.
-
-    testdir.makeini(
-        """
-        [pytest]
-        doctest_subpackage_requires =
-            test/a/* = pytest>1
-            test/b/*= pytest>1;averyfakepackage>99999.9
-            test/c/*=anotherfakepackage>=22000.1.2
+# Note that each entry under doctest_subpackage_requires has different whitespace
+# around the = to make sure that all cases work properly.
+SUBPACKAGE_REQUIRES_INI = (
+    "makeini",
     """
-    )
+    [pytest]
+    doctest_subpackage_requires =
+        test/a/* = pytest>1
+        test/b/*= pytest>1;averyfakepackage>99999.9
+        test/c/*=anotherfakepackage>=22000.1.2
+    """
+)
+SUBPACKAGE_REQUIRES_PYPROJECT = (
+    "makepyprojecttoml",
+    """
+    [tool.pytest.ini_options]
+    doctest_subpackage_requires = [
+        "test/a/* = pytest>1",
+        "test/b/*= pytest>1;averyfakepackage>99999.9",
+        "test/c/*=anotherfakepackage>=22000.1.2",
+    ]
+    """
+)
+
+
+@pytest.fixture()
+def subpackage_requires_testdir(testdir, request):
+    if request.param[0] == 'makepyprojecttoml' and PYTEST_LT_6:
+        return None, None
+
+    config_file = getattr(testdir, request.param[0])(request.param[1])
+
     test = testdir.mkdir('test')
     a = test.mkdir('a')
     b = test.mkdir('b')
@@ -801,12 +850,46 @@ def test_doctest_subpackage_requires(testdir, caplog):
     b.join('testcode.py').write(pyfile)
     c.join('testcode.py').write(pyfile)
 
-    reprec = testdir.inline_run(test, "--doctest-plus")
+    return config_file, testdir
+
+
+@pytest.mark.parametrize('subpackage_requires_testdir', [SUBPACKAGE_REQUIRES_INI, SUBPACKAGE_REQUIRES_PYPROJECT], indirect=True)
+def test_doctest_subpackage_requires(subpackage_requires_testdir, caplog):
+    config_file, testdir = subpackage_requires_testdir
+    if config_file is None:
+        pytest.skip("pyproject.toml not supported in pytest<6")
+
+    reprec = testdir.inline_run(str(testdir), f"-c={config_file}", "--doctest-plus")
     reprec.assertoutcome(passed=1)
     assert reprec.listoutcomes()[0][0].location[0] == os.path.join('test', 'a', 'testcode.py')
     assert caplog.text == ''
 
 
+@pytest.mark.parametrize(('import_mode', 'expected'), [
+    pytest.param('importlib', dict(passed=2), marks=pytest.mark.skipif(PYTEST_LT_6, reason="importlib import mode not supported on Pytest <6"), id="importlib"),
+    pytest.param('append', dict(failed=1), id="append"),
+    pytest.param('prepend', dict(failed=1), id="prepend"),
+])
+def test_import_mode(testdir, import_mode, expected):
+    """Test that two files with the same name but in different folders work with --import-mode=importlib."""
+    a = testdir.mkdir('a')
+    b = testdir.mkdir('b')
+
+    pyfile = dedent("""
+        def f():
+            '''
+            >>> 1
+            1
+            '''
+    """)
+
+    a.join('testcode.py').write(pyfile)
+    b.join('testcode.py').write(pyfile)
+
+    reprec = testdir.inline_run(str(testdir), "--doctest-plus", f"--import-mode={import_mode}")
+    reprec.assertoutcome(**expected)
+
+
 def test_doctest_skip(testdir):
     testdir.makeini(
         """
@@ -1039,6 +1122,24 @@ def test_fail_data_dependency(testdir, cont_on_fail):
     assert ("something()\nUNEXPECTED EXCEPTION: NameError" in report.longreprtext) is cont_on_fail
 
 
+def test_main(testdir):
+    pkg = testdir.mkdir('pkg')
+    code = dedent(
+        '''
+        def f():
+            raise RuntimeError("This is a CLI, do not execute module while doctesting")
+        
+        f()
+        '''
+    )
+    pkg.join('__init__.py').write_text("", "utf-8")
+    main_path = pkg.join('__main__.py')
+    main_path.write_text(code, "utf-8")
+
+    testdir.inline_run(pkg).assertoutcome(passed=0)
+    testdir.inline_run(pkg, '--doctest-plus').assertoutcome(passed=0)
+
+
 def test_ufunc(testdir):
     pytest.importorskip('numpy')
 
@@ -1144,6 +1245,88 @@ def test_ufunc(testdir):
     result.assertoutcome(passed=2, failed=0)
 
 
+NORCURSEDIRS_INI = (
+    "makeini",
+    """
+    [pytest]
+    doctest_norecursedirs =
+        "bad_dir"
+        "*/bad_file.py"
+    """
+)
+NORCURSEDIRS_PYPROJECT = (
+    "makepyprojecttoml",
+    """
+    [tool.pytest.ini_options]
+    doctest_norecursedirs = [
+        "bad_dir",
+        "*/bad_file.py",
+    ]
+    """
+)
+
+
+@pytest.fixture()
+def norecursedirs_testdir(testdir, request):
+    if request.param[0] == 'makepyprojecttoml' and PYTEST_LT_6:
+        return None, None
+
+    config_file = getattr(testdir, request.param[0])(request.param[1])
+
+    bad_text = dedent("""
+        def f():
+            '''
+            This should fail doc testing
+            >>> 1
+            2
+            '''
+            pass
+    """)
+
+    good_text = dedent("""
+        def g():
+            '''
+            This should pass doc testing
+            >>> 1
+            1
+            '''
+            pass
+    """)
+
+    # Create a bad file that should be by its folder
+    bad_subdir = testdir.mkdir("bad_dir")
+    bad_file = bad_subdir.join("test_foobar.py")
+    bad_file.write_text(bad_text, "utf-8")
+
+    # Create a bad file that should be skipped by its name
+    okay_subdir1 = testdir.mkdir("okay_foo_dir")
+    bad_file = okay_subdir1.join("bad_file.py")
+    bad_file.write_text(bad_text, "utf-8")  
+    # Create a good file in that directory that doctest won't skip
+    good_file1 = okay_subdir1.join("good_file1.py")
+    good_file1.write_text(good_text, "utf-8")
+
+    # Create another bad file that should be skipped by its name
+    okay_subdir2 = testdir.mkdir("okay_bar_dir")
+    bad_file = okay_subdir2.join("bad_file.py")
+    bad_file.write_text(bad_text, "utf-8")  
+    # Create a good file in that directory that doctest won't skip
+    good_file2 = okay_subdir2.join("good_file2.py")
+    good_file2.write_text(good_text, "utf-8")
+
+    return config_file, testdir
+
+
+@pytest.mark.parametrize('norecursedirs_testdir', [NORCURSEDIRS_INI, NORCURSEDIRS_PYPROJECT], indirect=True)
+def test_doctest_norecursedirs(norecursedirs_testdir):
+    config_file, testdir = norecursedirs_testdir
+    if config_file is None:
+        pytest.skip("pyproject.toml not supported in pytest<6")
+
+    reprec = testdir.inline_run(str(testdir), f"-c={config_file}", "--doctest-plus")
+    reprec.assertoutcome(passed=2)
+
+
 def test_norecursedirs(testdir):
     testdir.makeini(
         """
@@ -1165,3 +1348,39 @@ def test_norecursedirs(testdir):
     """, "utf-8")
     reprec = testdir.inline_run(str(testdir), "--doctest-plus")
     reprec.assertoutcome(failed=0, passed=0)
+
+
+def test_generate_diff_basic(testdir, capsys):
+    p = testdir.makepyfile("""
+        def f():
+            '''
+            >>> print(2)
+            4
+            >>> print(3)
+            5
+            '''
+            pass
+        """)
+    with open(p) as f:
+        original = f.read()
+
+    testdir.inline_run(p, "--doctest-plus-generate-diff")
+    diff = dedent("""
+         >>> print(2)
+    -    4
+    +    2
+         >>> print(3)
+    -    5
+    +    3
+    """)
+    captured = capsys.readouterr()
+    assert diff in captured.out
+
+    testdir.inline_run(p, "--doctest-plus-generate-diff=overwrite")
+    captured = capsys.readouterr()
+    assert "Applied fix to the following files" in captured.out
+
+    with open(p) as f:
+        result = f.read()
+
+    assert result == original.replace("4", "2").replace("5", "3")
